/* eslint-disable @typescript-eslint/no-explicit-any */
import { VegaWalletContext } from '@vegaprotocol/wallet';
import { act, render, screen, waitFor } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import { generateMarket, generateMarketData } from '../../test-helpers';
import { DealTicket } from './deal-ticket';
import * as Schema from '@vegaprotocol/types';
import type { MockedResponse } from '@apollo/client/testing';
import { MockedProvider } from '@apollo/client/testing';
import { addDecimal } from '@vegaprotocol/utils';
import type { OrdersQuery } from '@vegaprotocol/orders';
import {
  DealTicketType,
  useDealTicketFormValues,
} from '../../hooks/use-form-values';
import * as positionsTools from '@vegaprotocol/positions';
import { OrdersDocument } from '@vegaprotocol/orders';

jest.mock('zustand');
jest.mock('./deal-ticket-fee-details', () => ({
  DealTicketFeeDetails: () => <div data-testid="deal-ticket-fee-details" />,
  DealTicketMarginDetails: () => (
    <div data-testid="deal-ticket-margin-details" />
  ),
}));

const marketPrice = '200';
const pubKey = 'pubKey';
const market = generateMarket();
const marketData = generateMarketData();
const submit = jest.fn();

function generateJsx(mocks: MockedResponse[] = []) {
  return (
    <MockedProvider mocks={[...mocks]}>
      <VegaWalletContext.Provider value={{ pubKey, isReadOnly: false } as any}>
        <DealTicket
          market={market}
          marketData={marketData}
          marketPrice={marketPrice}
          submit={submit}
          onDeposit={jest.fn()}
        />
      </VegaWalletContext.Provider>
    </MockedProvider>
  );
}

describe('DealTicket', () => {
  beforeEach(() => {
    jest.clearAllMocks();
    localStorage.clear();
  });

  it('check filtering of active orders', async () => {
    const mockOrders: OrdersQuery = {
      party: {
        id: 'pubKey',
        ordersConnection: {
          edges: [
            {
              node: {
                id: 'order-id-1',
                remaining: '101010',
                market: {
                  id: 'market-id',
                },
              },
            },
            {
              node: {
                id: 'order-id-2',
                remaining: '1111',
                market: {
                  id: 'other-market-id',
                },
              },
            },
          ],
        },
      },
    } as unknown as OrdersQuery;
    const orderMocks = {
      request: {
        query: OrdersDocument,
        variables: {
          partyId: 'pubKey',
          filter: { liveOnly: true },
          pagination: { first: 5000 },
        },
      },
      result: {
        data: mockOrders,
      },
    };
    jest.spyOn(positionsTools, 'useEstimatePositionQuery');
    render(generateJsx([orderMocks]));
    await waitFor(() => {
      expect(screen.getByTestId('deal-ticket-fee-details')).toBeInTheDocument();
      expect(
        (positionsTools.useEstimatePositionQuery as jest.Mock).mock.lastCall[0]
          .variables.orders
      ).toHaveLength(2);
      expect(
        (positionsTools.useEstimatePositionQuery as jest.Mock).mock.lastCall[0]
          .variables.orders[0].remaining
      ).toEqual('101010');
      expect(
        (positionsTools.useEstimatePositionQuery as jest.Mock).mock.lastCall[0]
          .variables.orders[1].remaining
      ).toEqual('0');
    });
  });

  it('should display ticket defaults', () => {
    render(generateJsx());

    // place order button should always be enabled
    expect(screen.getByTestId('place-order')).toBeEnabled();

    // Assert defaults are used
    expect(screen.getByTestId('order-type-Market')).toBeInTheDocument();
    expect(screen.getByTestId('order-type-Limit')).toBeInTheDocument();

    expect(screen.getByTestId('order-type-Limit').dataset.state).toEqual(
      'checked'
    );

    expect(screen.getByTestId('order-side-SIDE_BUY').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-size')).toHaveDisplayValue('0');
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_GTC
    );
  });

  it('should display last price for market type order', () => {
    render(generateJsx());
    act(() => {
      screen.getByTestId('order-type-Market').click();
    });
    // Assert last price is shown
    expect(screen.getByTestId('last-price')).toHaveTextContent(
      // eslint-disable-next-line
      `~${addDecimal(marketPrice, market.decimalPlaces)} ${
        market.tradableInstrument.instrument.product.quoteName
      }`
    );
  });

  it('should use local storage state for initial values', () => {
    const expectedOrder = {
      marketId: market.id,
      type: Schema.OrderType.TYPE_LIMIT,
      side: Schema.Side.SIDE_SELL,
      size: '0.1',
      price: '300.22',
      timeInForce: Schema.OrderTimeInForce.TIME_IN_FORCE_IOC,
      persist: true,
    };

    useDealTicketFormValues.setState({
      formValues: {
        [expectedOrder.marketId]: {
          [DealTicketType.Limit]: expectedOrder,
        },
      },
    });

    render(generateJsx());

    // Assert correct defaults are used from store
    expect(screen.getByTestId('order-type-Limit').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-side-SIDE_SELL').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-size')).toHaveDisplayValue(
      expectedOrder.size
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      expectedOrder.timeInForce
    );
    expect(screen.getByTestId('order-price')).toHaveDisplayValue(
      expectedOrder.price
    );
  });

  it('should set values for a non-persistent reduce only order and disable post only checkbox', () => {
    const expectedOrder = {
      marketId: market.id,
      type: Schema.OrderType.TYPE_LIMIT,
      side: Schema.Side.SIDE_SELL,
      size: '0.1',
      price: '300.22',
      timeInForce: Schema.OrderTimeInForce.TIME_IN_FORCE_IOC,
      persist: false,
      reduceOnly: true,
      postOnly: false,
    };
    useDealTicketFormValues.setState({
      formValues: {
        [expectedOrder.marketId]: {
          [DealTicketType.Limit]: expectedOrder,
        },
      },
    });

    render(generateJsx());

    // Assert correct defaults are used from store
    expect(screen.getByTestId('order-type-Limit').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-side-SIDE_SELL').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-size')).toHaveDisplayValue(
      expectedOrder.size
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      expectedOrder.timeInForce
    );
    expect(screen.getByTestId('order-price')).toHaveDisplayValue(
      expectedOrder.price
    );
    expect(screen.getByTestId('post-only')).toBeDisabled();
    expect(screen.getByTestId('reduce-only')).toBeEnabled();
    expect(screen.getByTestId('reduce-only')).toBeChecked();
    expect(screen.getByTestId('post-only')).not.toBeChecked();
  });

  it('should set values for a persistent post only order and disable reduce only checkbox', () => {
    const expectedOrder = {
      marketId: market.id,
      type: Schema.OrderType.TYPE_LIMIT,
      side: Schema.Side.SIDE_SELL,
      size: '0.1',
      price: '300.22',
      timeInForce: Schema.OrderTimeInForce.TIME_IN_FORCE_GTC,
      persist: true,
      reduceOnly: false,
      postOnly: true,
    };

    useDealTicketFormValues.setState({
      formValues: {
        [expectedOrder.marketId]: {
          [DealTicketType.Limit]: expectedOrder,
        },
      },
    });

    render(generateJsx());

    // Assert correct defaults are used from store
    expect(screen.getByTestId('order-type-Limit').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-side-SIDE_SELL').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-size')).toHaveDisplayValue(
      expectedOrder.size
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      expectedOrder.timeInForce
    );
    expect(screen.getByTestId('order-price')).toHaveDisplayValue(
      expectedOrder.price
    );
    expect(screen.getByTestId('post-only')).toBeEnabled();
    expect(screen.getByTestId('reduce-only')).toBeDisabled();
    expect(screen.getByTestId('post-only')).toBeChecked();
    expect(screen.getByTestId('reduce-only')).not.toBeChecked();
  });

  it('should set values for a persistent post only iceberg order and disable reduce only checkbox', () => {
    const expectedOrder = {
      marketId: market.id,
      type: Schema.OrderType.TYPE_LIMIT,
      side: Schema.Side.SIDE_SELL,
      size: '10',
      price: '300.22',
      timeInForce: Schema.OrderTimeInForce.TIME_IN_FORCE_GTC,
      persist: true,
      reduceOnly: false,
      postOnly: true,
      iceberg: true,
      icebergOpts: {
        peakSize: '5',
        minimumVisibleSize: '7',
      },
    };

    useDealTicketFormValues.setState({
      formValues: {
        [expectedOrder.marketId]: {
          [DealTicketType.Limit]: expectedOrder,
        },
      },
    });

    render(generateJsx());

    // Assert correct defaults are used from store
    expect(screen.getByTestId('order-type-Limit').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-side-SIDE_SELL').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-size')).toHaveDisplayValue(
      expectedOrder.size
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      expectedOrder.timeInForce
    );
    expect(screen.getByTestId('order-price')).toHaveDisplayValue(
      expectedOrder.price
    );
    expect(screen.getByTestId('post-only')).toBeEnabled();
    expect(screen.getByTestId('reduce-only')).toBeDisabled();
    expect(screen.getByTestId('post-only')).toBeChecked();
    expect(screen.getByTestId('reduce-only')).not.toBeChecked();
    expect(screen.getByTestId('iceberg')).toBeEnabled();
    expect(screen.getByTestId('iceberg')).toBeChecked();
  });

  it('should set values for a non-persistent iceberg order and disable post only checkbox', () => {
    const expectedOrder = {
      marketId: market.id,
      type: Schema.OrderType.TYPE_LIMIT,
      side: Schema.Side.SIDE_SELL,
      size: '0.1',
      price: '300.22',
      timeInForce: Schema.OrderTimeInForce.TIME_IN_FORCE_IOC,
      persist: false,
      reduceOnly: false,
      postOnly: false,
    };
    useDealTicketFormValues.setState({
      formValues: {
        [expectedOrder.marketId]: {
          [DealTicketType.Limit]: expectedOrder,
        },
      },
    });

    render(generateJsx());

    // Assert correct defaults are used from store
    expect(screen.getByTestId('order-type-Limit').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-side-SIDE_SELL').dataset.state).toEqual(
      'checked'
    );
    expect(screen.getByTestId('order-size')).toHaveDisplayValue(
      expectedOrder.size
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      expectedOrder.timeInForce
    );
    expect(screen.getByTestId('order-price')).toHaveDisplayValue(
      expectedOrder.price
    );
    expect(screen.getByTestId('post-only')).toBeDisabled();
    expect(screen.getByTestId('reduce-only')).toBeEnabled();
    expect(screen.getByTestId('reduce-only')).not.toBeChecked();
    expect(screen.getByTestId('post-only')).not.toBeChecked();
    expect(screen.getByTestId('iceberg')).not.toBeChecked();
  });

  // eslint-disable-next-line jest/no-disabled-tests
  it('handles TIF select box dependent on order type', async () => {
    render(generateJsx());

    act(() => {
      screen.getByTestId('order-type-Market').click();
    });

    // Only FOK and IOC should be present for type market order
    expect(
      Array.from(screen.getByTestId('order-tif').children).map(
        (o) => o.textContent
      )
    ).toEqual(['Fill or Kill (FOK)', 'Immediate or Cancel (IOC)']);

    // IOC should be default
    expect(screen.getByTestId('order-tif')).toHaveDisplayValue(
      'Immediate or Cancel (IOC)'
    );

    // Select FOK - FOK should be selected
    await userEvent.selectOptions(
      screen.getByTestId('order-tif'),
      Schema.OrderTimeInForce.TIME_IN_FORCE_FOK
    );
    expect(screen.getByTestId('order-tif')).toHaveDisplayValue(
      'Fill or Kill (FOK)'
    );

    // Switch to type limit order -> all TIF options should be shown
    await userEvent.click(screen.getByTestId('order-type-Limit'));
    expect(screen.getByTestId('order-tif').children).toHaveLength(
      Object.keys(Schema.OrderTimeInForce).length
    );

    // expect GTC as LIMIT default
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_GTC
    );

    // Select GTT -> GTT should be selected
    await userEvent.selectOptions(
      screen.getByTestId('order-tif'),
      Schema.OrderTimeInForce.TIME_IN_FORCE_GTT
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_GTT
    );

    // Switch back to type market order -> FOK should be preserved from previous selection
    await userEvent.click(screen.getByTestId('order-type-Market'));
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_FOK
    );

    // Select IOC -> IOC should be selected
    await userEvent.selectOptions(
      screen.getByTestId('order-tif'),
      Schema.OrderTimeInForce.TIME_IN_FORCE_IOC
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_IOC
    );

    // Switch back type limit order -> GTT should be preserved
    await userEvent.click(screen.getByTestId('order-type-Limit'));
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_GTT
    );

    // Select GFN -> GFN should be selected
    await userEvent.selectOptions(
      screen.getByTestId('order-tif'),
      Schema.OrderTimeInForce.TIME_IN_FORCE_GFN
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_GFN
    );

    // Switch to type market order -> IOC should be preserved
    await userEvent.click(screen.getByTestId('order-type-Market'));
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_IOC
    );
  });

  it('can edit deal ticket', async () => {
    render(generateJsx());

    // BUY is selected by default
    expect(screen.getByTestId('order-side-SIDE_BUY').dataset.state).toEqual(
      'checked'
    );

    await userEvent.type(screen.getByTestId('order-size'), '200');

    expect(screen.getByTestId('order-size')).toHaveDisplayValue('200');

    await userEvent.selectOptions(
      screen.getByTestId('order-tif'),
      Schema.OrderTimeInForce.TIME_IN_FORCE_IOC
    );
    expect(screen.getByTestId('order-tif')).toHaveValue(
      Schema.OrderTimeInForce.TIME_IN_FORCE_IOC
    );

    // Switch to limit order
    await userEvent.click(screen.getByTestId('order-type-Limit'));

    // Check all TIF options shown
    expect(screen.getByTestId('order-tif').children).toHaveLength(
      Object.keys(Schema.OrderTimeInForce).length
    );
  });
});
